#
# MIT License
#
# Copyright (c) 2024 nbiotcloud
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

"""Unified Chip Design Platform - IPXACT."""

from logging import getLogger

import ucdp as u
from tabulate import tabulate
from ucdp_addr import AddrMap, Addrspace

LOGGER = getLogger(__name__)


class UcdpIpxact(u.Object):
    """UCDP Representation of IPXACT."""

    vendor: str
    version: str
    libname: str
    name: str
    ports: tuple[u.Port, ...] = ()
    addrspaces: tuple[Addrspace, ...] = ()
    filelists: u.ModFileLists = ()

    def get_overview(self) -> str:
        """Nice Overview."""
        addrmap = AddrMap.from_addrspaces(self.addrspaces)
        ports = [(port.direction, port.name, port.type_) for port in self.ports]
        data = (
            ("Attribute", "Value"),
            ("vendor", self.vendor),
            ("version", self.version),
            ("libname", self.libname),
            ("name", self.name),
            ("ports", tabulate(ports, headers=("I/O", "Name", "Type"))),
            ("addrmap", addrmap.get_overview()),
        )
        return tabulate(data, tablefmt="fancy_grid") + "\n"
